/*!
 * Copyright (c) 2021 Eclipse Foundation, Inc.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * Contributors:
 *   Eric Poirier <eric.poirier@eclipse-foundation.org>
 *
 * SPDX-License-Identifier: EPL-2.0
 */

import $, { type } from 'jquery';
import Mustache from 'mustache';
import template from './templates/tpl-projects-item.mustache';
import "numeral";
import List from 'list.js';

export const processJSON = (json, options) => {

    function fileExists(url) {
        if(url){
            let req = new XMLHttpRequest();
            req.open('GET', url, false);
            req.send();
            return req.status == 200;
        } else {
            return false;
        }
    }

    // Initiate the projects array
    let projects = json;

    // Filter by project ID
    if (options.projectIds !== undefined && options.projectIds !== "") {
        const acceptedProjectIds = options.projectIds.split(",");
        projects = [];
        $(json).each(function(key, element){
            if (acceptedProjectIds.includes(element.project_id)) {
                projects.push(element);
            }
        });
    }

    $(projects).each(function(key, project){
        projects[key].version = 'none';
        if (project.releases[0]) {
            projects[key].version = project.releases[0].name;
        }

        if (options.templateId !== "tpl-projects-item") {
            $(project.project_leads).each(function(project_lead_key, project_lead){
                projects[key].project_lead_image = '/images/ecdtools/home/silhouette.jpg';
                projects[key].project_lead_full_name = project_lead.full_name;
                if (fileExists('/images/ecdtools/projects/'+ project_lead.username +'.png')) {
                    projects[key].project_lead_image = '/images/ecdtools/projects/'+ project_lead.username +'.png';
                }
                return false;
            });
        }
    });

    return projects;
};

(function ($, document) {
    
    $('.featured-projects').each(function (index, element) {
        const options = {
            id: '',
            templateId: '',
            projectIds: '',
            url: '',
            ...$(element).data(),
        };

        // fetching the projects
        function fetchProjects() {
            return fetch(options.url).then(response => response.json())
        }

        function fetchCategories(){
            let elem = document.getElementById('projects-categories');
            if (elem !== null){
                let categoriesUrl = elem.getAttribute('data-categories');
                return fetch(categoriesUrl).then(response => response.json());
            }

            // returning an empty promise if categories were not passed.
            return async () => {};
        }

        let resultsProjects = [];
        let resultsCategories = [];
        Promise.allSettled([fetchProjects(), fetchCategories()])
        .then(function (responses) {
            if (responses[0].status === 'fulfilled') {
                const projects = processJSON(responses[0].value, options);
                if (projects.length > 0) {
                    resultsProjects = projects;
                }
            }

            if (responses[1].status === 'fulfilled') {
                resultsCategories = responses[1].value;
            }

            $(resultsProjects).each(function(key, project){

                // Populate the Tags array
                if (resultsCategories.length <= 0 && typeof resultsProjects[key].tags !== "undefined"){
                    const project_tags = resultsProjects[key].tags;
                    if (project_tags.length !== 0) {
                        tags[resultsProjects[key].project_id] = project_tags;
                    }
                }

                const category = getCategory(resultsCategories, resultsProjects[key].project_id);
                resultsProjects[key].category = category.toString();
            });

            const data = {
                items: resultsProjects,
            };

            // Render component
            let html = '';
            if (options.templateId !== '' && document.getElementById(options.templateId)) {
                const theme = document.getElementById(options.templateId).innerHTML;
                html = Mustache.render(theme, data);
            } else {
                html = template(data);
            }
            element.innerHTML += html;

            if (resultsCategories.length <= 0) {
                resultsCategories = tags;
            }

            $.each( resultsCategories, function( key, projects ) {
                $.each( projects, function( key, value ) {
                    let duplicate = $('.eclipsefdn-project-list-filters').find("button:contains('"+ value +"')");
                    if (duplicate.length > 0) {
                        return;
                    }
                    let button = document.createElement('button');
                    button.innerHTML = value;
                    button.className = 'btn btn-filter-project';
                    button.setAttribute('data-toggle','button');
                    $('.eclipsefdn-project-list-filters').append(button);
                });
            });

            const values = {
                valueNames: [
                  'name',
                  'category'
                ],
            };

            let list = new List('projects-list', values);

            $('.btn-filter-project').on('click', function (elem) {
                $('.btn-filter-project').not(this).each(function(){
                    $(this).removeClass('active');
                });
                setTimeout(function () {
                    list.filter(computeFilterFunction);
                }, 10);
            });
        })
        .catch(err => console.log(err));
    });
})(jQuery, document);

let tags = {};

let getCategory = function(categories, project_id) {
    let category = "Other Tools";
    if (typeof categories[project_id] !== "undefined") {
        category = categories[project_id];
    }
    if (categories.length <= 0 && typeof tags[project_id] !== "undefined") {
        category = tags[project_id].toString().split(',').join(', ');
    }
    return category;
}

let computeFilterFunction = function (item) {
    let filter = [];
    $('.btn-filter-project').each(function (index, elem) {
        if ($(elem).hasClass('active')) {
            filter.push($(elem).text());
        }
    });
    if (filter.length == 0) {
        return true;
    }
    let found = false;
    for (let i = 0; i < filter.length; i++) {
        const element = filter[i];       
        if (typeof item.values().category !== "undefined" && item.values().category.indexOf(element) !== -1) {
            found = true;
            continue;
        }
        found = false;
        break;
    }
    return found;
};
